# -*- coding: utf-8 -*-
## $Id: test_browser.py 12840 2015-04-01 08:35:16Z henrik $
##
## Copyright (C) 2015 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""
Extends the doctests in browser.txt and datacenter.txt.
"""
import shutil
import tempfile
import os
import grok
from datetime import datetime, timedelta
from zc.async.testing import wait_for_result
from zope.component import createObject, getUtility
from zope.component.hooks import setSite, clearSite
from zope.security.interfaces import Unauthorized
from zope.securitypolicy.interfaces import IPrincipalRoleManager
from zope.testbrowser.testing import Browser
from waeup.ikoba.testing import FunctionalLayer, FunctionalTestCase
from waeup.ikoba.app import Company
from waeup.ikoba.interfaces import IJobManager, IUserAccount, IIkobaUtils
from waeup.ikoba.tests.test_async import FunctionalAsyncTestCase


class CompanySetup(FunctionalTestCase):
    # A test case that only contains a setup and teardown

    layer = FunctionalLayer

    def setUp(self):
        super(CompanySetup, self).setUp()

        # Setup a sample site for each test
        app = Company()
        self.dc_root = tempfile.mkdtemp()
        app['datacenter'].setStoragePath(self.dc_root)

        # Prepopulate the ZODB...
        self.getRootFolder()['app'] = app
        # we add the site immediately after creation to the
        # ZODB. Catalogs and other local utilities are not setup
        # before that step.
        self.app = self.getRootFolder()['app']
        # Set site here. Some of the following setup code might need
        # to access grok.getSite() and should get our new app then
        setSite(app)

        # Prepare test browser
        self.browser = Browser()
        self.browser.handleErrors = False

        self.datacenter_path = 'http://localhost/app/datacenter'

    def tearDown(self):
        super(CompanySetup, self).tearDown()
        clearSite()
        shutil.rmtree(self.dc_root)


class DataCenterUITests(CompanySetup, FunctionalAsyncTestCase):
    # Tests for DataCenter class views and pages

    layer = FunctionalLayer

    def wait_for_export_job_completed(self):
        # helper function waiting until the current export job is completed
        manager = getUtility(IJobManager)
        job_id = self.app['datacenter'].running_exports[0][0]
        job = manager.get(job_id)
        wait_for_result(job)
        return job_id

    def trigger_export(self):
        # helper to start an export. Make sure to remove the result file
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.datacenter_path)
        self.browser.getLink("Export data").click()
        self.browser.getControl(name="exporter").value = ['products']
        self.browser.getControl("Create CSV file").click()
        return

    def cleanup_run_job(self):
        # helper to remove any result file of an export
        job_id = self.wait_for_export_job_completed()
        manager = getUtility(IJobManager)
        job = manager.get(job_id)
        if os.path.exists(job.result):
            shutil.rmtree(os.path.dirname(job.result))

    def test_logs_listed(self):
        # existing logfiles are listed in logs overview
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        log_path = os.path.join(
            self.app['datacenter'].storage, 'logs', 'myspecial.log')
        with open(log_path, "a") as fd:
            fd.write("SOME FOO IN LOGFILE")
        self.browser.open(self.datacenter_path + "/logs")
        assert "myspecial.log" in self.browser.contents

    def test_view_log(self):
        # single logfiles can be watched (with filter regexp)
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        log_path = os.path.join(
            self.app['datacenter'].storage, 'logs', 'myspecial.log')
        with open(log_path, "a") as fd:
            fd.write("SOME FOO IN LOGFILE")
        self.browser.open(
            self.datacenter_path + "/@@show/?logname=myspecial.log&query=.*")
        assert "SOME FOO IN LOGFILE" in self.browser.contents

    def test_html_in_logs_escaped(self):
        # HTML code in logfiles is escaped before output.
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        log_path = os.path.join(
            self.app['datacenter'].storage, 'logs', 'myspecial.log')
        with open(log_path, "a") as fd:
            fd.write("SOME <TAG> AND ÜMLÄUTS IN LOG")
        self.browser.open(
            self.datacenter_path + "/@@show/?logname=myspecial.log&query=.*")
        assert "SOME &lt;TAG&gt; AND ÜMLÄUTS IN LOG" in self.browser.contents

    def test_user_data_import_permission(self):
        upload_path = 'http://localhost/app/datacenter/upload'
        login_path = 'http://localhost/app/login'
        # Create  officer
        self.app['users'].addUser('officer', 'officersecret')
        self.app['users']['officer'].email = 'mrofficer@foo.ng'
        self.app['users']['officer'].title = 'Carlo Pitter'
        # Assign role
        prm = IPrincipalRoleManager(self.app)
        prm.assignRoleToPrincipal('waeup.ImportManager', 'officer')
        # Login
        self.browser.open(login_path)
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'officersecret'
        self.browser.getControl("Login").click()
        self.browser.open(upload_path)
        self.assertFalse('User Processor' in self.browser.contents)
        # Assign role which is necessary to import user data
        prm.assignRoleToPrincipal('waeup.UsersManager', 'officer')
        self.browser.open(upload_path)
        self.assertTrue('User Processor' in self.browser.contents)
        return

    def test_export_disabled(self):
        self.app['configuration'].export_disabled_message = u'Export forbidden'
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.datacenter_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.datacenter_path)
        self.browser.getLink("Export data").click()
        self.browser.getControl(name="exporter").value = ['products']
        self.browser.getControl("Create CSV file").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue('div class="alert alert-danger">Export forbidden</div>'
                         in self.browser.contents)
        return

    def test_export_start(self):
        # we can trigger export file creation
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open(self.datacenter_path)
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.url, self.datacenter_path)
        self.browser.getLink("Export data").click()
        self.browser.getControl(name="exporter").value = ['products']
        self.browser.getControl("Create CSV file").click()
        # we can trigger export file creation only once
        self.browser.getControl(name="exporter").value = ['pdfdocuments']
        self.browser.getControl("Create CSV file").click()
        self.assertTrue('Sorry, another export job is running.'
                         in self.browser.contents)
        self.wait_for_export_job_completed()
        self.assertEqual(len(self.app['datacenter'].running_exports), 1)
        self.browser.getControl(name="exporter").value = ['pdfdocuments']
        self.browser.getControl("Create CSV file").click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(len(self.app['datacenter'].running_exports), 2)
        # The second job is running
        self.assertTrue('<img src="/static/img/actionicon_reload.png" />'
                         in self.browser.contents)
        self.cleanup_run_job() # make sure to remove temp dirs
        # Also the second job must be cleaned up
        manager = getUtility(IJobManager)
        job_id = self.app['datacenter'].running_exports[1][0]
        job = manager.get(job_id)
        wait_for_result(job)
        if os.path.exists(job.result):
            shutil.rmtree(os.path.dirname(job.result))
        return

    def test_export_download(self):
        # we can download a generated export result
        self.trigger_export()
        # while the export file is created, we get a reload button
        # (or a loading bar if javascript is enabled)...
        self.assertTrue('Reload' in self.browser.contents)
        # ...which is displayed as long as the job is not finished.
        # When the job is finished and we reload the page...
        job_id = self.wait_for_export_job_completed()
        self.browser.open(self.datacenter_path + '/export')
        self.assertFalse('Reload' in self.browser.contents)
        # ...we can download the result
        self.browser.getLink("Download").click()
        self.assertEqual(self.browser.headers['content-type'],
                         'text/csv; charset=UTF-8')
        self.assertEqual(self.browser.headers['content-disposition'],
            'attachment; filename="WAeUP.Ikoba_products_%s.csv' % job_id)
        self.assertEqual(self.browser.contents,
            'contract_category,contract_title,description,options,'
            'product_id,terms_and_conditions,title,valid_from,valid_to,'
            'users_with_local_roles\r\n')
        # Thew job can be discarded
        self.browser.open(self.datacenter_path + '/export')
        self.browser.getControl("Discard").click()
        self.assertEqual(len(self.app['datacenter'].running_exports), 0)
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'datacenter.log')
        logcontent = open(logfile).read()
        self.assertTrue('zope.mgr - browser.pages.ExportCSVPage - exported: '
                        'products, job_id=' in logcontent)
        self.assertTrue('zope.mgr - browser.pages.ExportCSVView - '
                        'downloaded: WAeUP.Ikoba_products_%s.csv, job_id=%s'
                        % (job_id, job_id) in logcontent)
        return

    def test_export_discard(self):
        # we can discard a generated export result
        self.trigger_export()
        job_id = self.wait_for_export_job_completed()
        self.browser.open(self.datacenter_path + '/@@export')
        self.browser.getControl("Discard").click()
        self.assertTrue('Discarded export' in self.browser.contents)
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'datacenter.log')
        logcontent = open(logfile).read()
        self.assertTrue(
            'zope.mgr - browser.pages.ExportCSVPage - discarded: job_id=%s'
            % job_id in logcontent)
        return

    def test_export_purge(self):
        self.trigger_export()
        job_id = self.wait_for_export_job_completed()
        self.browser.open(self.datacenter_path + '/export')
        self.browser.getControl("Purge").click()
        # Flash message always says successful ...
        self.assertTrue(
            'Exports successfully purged.' in self.browser.contents)
        # ... but the export is still there
        self.assertEqual(len(self.app['datacenter'].running_exports), 1)

        job_id, gen_name, user = self.app['datacenter'].get_running_export_jobs()[0]
        job = getUtility(IJobManager).get(job_id)
        tz = getUtility(IIkobaUtils).tzinfo
        delta = timedelta(days=5)
        setattr(job, '_begin_after', datetime.now(tz) - delta)
        self.browser.getControl("Purge").click()
        self.assertEqual(len(self.app['datacenter'].running_exports), 0)
        logfile = os.path.join(
            self.app['datacenter'].storage, 'logs', 'datacenter.log')
        logcontent = open(logfile).read()
        self.assertTrue('zope.mgr - browser.pages.ExportCSVPage - '
                        'purged: job_id=%s' % job_id in logcontent)
        return


class SupplementaryBrowserTests(CompanySetup):
    # These are additional tests to browser.txt

    def test_suspended_officer(self):
        self.app['users'].addUser(
            'officer', 'secret', title='Bob Officer', email='aa@aa.ng')
        # Officer can't login if their password is not set
        self.app['users']['officer'].password = None
        self.browser.open('http://localhost/app/login')
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'secret'
        self.browser.getControl("Login").click()
        self.assertTrue(
            'You entered invalid credentials.' in self.browser.contents)
        # We set the password again
        IUserAccount(
            self.app['users']['officer']).setPassword('secret')
        # Officers can't login if their account is suspended/deactivated
        self.app['users']['officer'].suspended = True
        self.browser.open('http://localhost/app/login')
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'secret'
        self.browser.getControl("Login").click()
        self.assertMatches(
            '...but yor account has been temporarily deactivated...',
            self.browser.contents)
        self.assertFalse("Bob Officer" in self.browser.contents)
        self.app['users']['officer'].suspended = False
        self.browser.open('http://localhost/app/login')
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'secret'
        self.browser.getControl("Login").click()
        self.assertMatches(
            '...You logged in...', self.browser.contents)
        self.assertTrue("Bob Officer" in self.browser.contents)
        self.browser.getLink("Logout").click()
        # Suspended accounts are marked
        self.browser.addHeader('Authorization', 'Basic mgr:mgrpw')
        self.browser.open('http://localhost/app/users')
        self.assertFalse('(suspended)' in self.browser.contents)
        self.app['users']['officer'].suspended = True
        self.browser.open('http://localhost/app/users')
        self.assertTrue(
            '<span style="color:red">(suspended)</span>'
            in self.browser.contents)
        self.browser.open('http://localhost/app/users/officer')
        self.assertTrue(
            'This account has been suspended.' in self.browser.contents)
        self.app['users']['officer'].suspended = False
        self.browser.open('http://localhost/app/users/officer')
        self.assertFalse(
            'This account has been suspended.' in self.browser.contents)
        return

    def test_maintenance_mode(self):
        config = grok.getSite()['configuration']
        self.app['users'].addUser(
            'officer', 'secret', title='Bob Officer', email='bob@abcd.ng')
        self.browser.open('http://localhost/app/login')
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'secret'
        self.browser.getControl("Login").click()
        # Officer logged in.
        self.assertMatches('...You logged in...', self.browser.contents)
        self.assertTrue("Bob Officer" in self.browser.contents)
        # If maintenance mode is enabled, officer is immediately logged out.
        config.maintmode_enabled_by = u'any_user'
        self.assertRaises(
            Unauthorized, self.browser.open, 'http://localhost/app/customers')
        self.browser.open('http://localhost/app/login')
        self.assertMatches(
            '...The portal is in maintenance mode...',
            self.browser.contents)
        # Officers really can't login if maintenance mode is enabled.
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'secret'
        self.browser.getControl("Login").click()
        # A second warning is raised.
        self.assertMatches(
            '...The portal is in maintenance mode. You can\'t login!...',
            self.browser.contents)
        # Offficer can login if s/he is the blocker.
        config.maintmode_enabled_by = u'officer'
        self.browser.getControl(name="form.login").value = 'officer'
        self.browser.getControl(name="form.password").value = 'secret'
        self.browser.getControl("Login").click()
        self.assertTrue('You logged in' in self.browser.contents)
        self.assertTrue('Logout' in self.browser.contents)
        return

    def test_sources_overview(self):
        self.browser.open('http://localhost/app/sources')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertTrue('id="headingCustomerStates"' in self.browser.contents)

    def test_processors_overview(self):
        self.browser.open('http://localhost/app/processors')
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.browser.getLink(url='skeleton?name=productprocessor').click()
        self.assertEqual(self.browser.headers['Status'], '200 Ok')
        self.assertEqual(self.browser.headers['Content-Type'],
                         'text/csv; charset=UTF-8')
        self.assertEqual(self.browser.contents,
            'contract_category,contract_title,description,'
            'options,product_id,terms_and_conditions,title,'
            'valid_from,valid_to\r\n')
        return